@@ -1,5 +1,6 @@
 #!/usr/bin/env php
 <?php
+// $Id: gen_curlinfo.php 323304 2012-02-17 21:13:24Z mike $
 
 error_reporting(0);
 
@@ -60,31 +61,29 @@ $translate = array(
 $templates = array(
 'STRING' => 
 '	if (CURLE_OK == curl_easy_getinfo(ch, %s, &c)) {
-		ZVAL_STRING(&tmp, STR_PTR(c));
-		zend_hash_str_update(info, "%s", lenof("%2$s"), &tmp);
+		add_assoc_string_ex(&array, "%s", sizeof("%2$s"), c ? c : "", 1);
 	}
 ',
 'DOUBLE' => 
 '	if (CURLE_OK == curl_easy_getinfo(ch, %s, &d)) {
-		ZVAL_DOUBLE(&tmp, d);
-		zend_hash_str_update(info, "%s", lenof("%2$s"), &tmp);
+		add_assoc_double_ex(&array, "%s", sizeof("%2$s"), d);
 	}
 ',
 'LONG' => 
 '	if (CURLE_OK == curl_easy_getinfo(ch, %s, &l)) {
-		ZVAL_LONG(&tmp, l);
-		zend_hash_str_update(info, "%s", lenof("%2$s"), &tmp);
+		add_assoc_long_ex(&array, "%s", sizeof("%2$s"), l);
 	}
 ',
 'SLIST' =>
 '	if (CURLE_OK == curl_easy_getinfo(ch, %s, &s)) {
-		array_init(&tmp);
+		MAKE_STD_ZVAL(subarray);
+		array_init(subarray);
 		for (p = s; p; p = p->next) {
 			if (p->data) {
-				add_next_index_string(&tmp, p->data);
+				add_next_index_string(subarray, p->data, 1);
 			}
 		}
-		zend_hash_str_update(info, "%s", lenof("%2$s"), &tmp);
+		add_assoc_zval_ex(&array, "%s", sizeof("%2$s"), subarray);
 		curl_slist_free_all(s);
 	}
 ',
