@@ -13,8 +13,7 @@
 #include "php.h"
 #include "php_http_buffer.h"
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_init_ex(
-		php_http_buffer_t *buf, size_t chunk_size, unsigned flags)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_init_ex(php_http_buffer_t *buf, size_t chunk_size, int flags)
 {
 	if (!buf) {
 		buf = pemalloc(sizeof(*buf), flags & PHP_HTTP_BUFFER_INIT_PERSISTENT);
@@ -23,8 +22,7 @@ PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_init_ex(
 	if (buf) {
 		buf->size = (chunk_size) ? chunk_size : PHP_HTTP_BUFFER_DEFAULT_SIZE;
 		buf->pmem = (flags & PHP_HTTP_BUFFER_INIT_PERSISTENT) ? 1 : 0;
-		buf->data = (flags & PHP_HTTP_BUFFER_INIT_PREALLOC) ?
-				pemalloc(buf->size, buf->pmem) : NULL;
+		buf->data = (flags & PHP_HTTP_BUFFER_INIT_PREALLOC) ? pemalloc(buf->size, buf->pmem) : NULL;
 		buf->free = (flags & PHP_HTTP_BUFFER_INIT_PREALLOC) ? buf->size : 0;
 		buf->used = 0;
 	}
@@ -32,13 +30,12 @@ PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_init_ex(
 	return buf;
 }
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_from_string_ex(
-		php_http_buffer_t *buf, const char *str, size_t len)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_from_string_ex(php_http_buffer_t *buf, const char *string, size_t length)
 {
 	int free_buf = !!buf;
 
-	if (EXPECTED(buf = php_http_buffer_init(buf))) {
-		if (PHP_HTTP_BUFFER_NOMEM == php_http_buffer_append(buf, str, len)) {
+	if ((buf = php_http_buffer_init(buf))) {
+		if (PHP_HTTP_BUFFER_NOMEM == php_http_buffer_append(buf, string, length)) {
 			if (free_buf) {
 				pefree(buf, buf->pmem);
 			}
@@ -48,9 +45,7 @@ PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_from_string_ex(
 	return buf;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_resize_ex(
-		php_http_buffer_t *buf, size_t len, size_t override_size,
-		zend_bool allow_error)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_resize_ex(php_http_buffer_t *buf, size_t len, size_t override_size, int allow_error)
 {
 	char *ptr = NULL;
 #if 0
@@ -59,13 +54,12 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_resize_ex(
 	if (buf->free < len) {
 		size_t size = override_size ? override_size : buf->size;
 		
-		while (UNEXPECTED((size + buf->free) < len)) {
+		while ((size + buf->free) < len) {
 			size <<= 1;
 		}
 		
 		if (allow_error) {
-			ptr = perealloc_recoverable(buf->data,
-					buf->used + buf->free + size, buf->pmem);
+			ptr = perealloc_recoverable(buf->data, buf->used + buf->free + size, buf->pmem);
 		} else {
 			ptr = perealloc(buf->data, buf->used + buf->free + size, buf->pmem);
 		}
@@ -82,8 +76,7 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_resize_ex(
 	return 0;
 }
 
-PHP_HTTP_BUFFER_API char *php_http_buffer_account(
-		php_http_buffer_t *buf, size_t to_account)
+PHP_HTTP_BUFFER_API char *php_http_buffer_account(php_http_buffer_t *buf, size_t to_account)
 {
 	assert(to_account <= buf->free);
 
@@ -109,24 +102,18 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_shrink(php_http_buffer_t *buf)
 	return buf->used;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_append(php_http_buffer_t *buf,
-		const char *append, size_t append_len)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_append(php_http_buffer_t *buf, const char *append, size_t append_len)
 {
-	if (	buf->free < append_len &&
-			PHP_HTTP_BUFFER_NOMEM == php_http_buffer_resize(buf, append_len)
-	) {
+	if (buf->free < append_len && PHP_HTTP_BUFFER_NOMEM == php_http_buffer_resize(buf, append_len)) {
 		return PHP_HTTP_BUFFER_NOMEM;
 	}
-	if (append_len) {
-		memcpy(buf->data + buf->used, append, append_len);
-		buf->used += append_len;
-		buf->free -= append_len;
-	}
+	memcpy(buf->data + buf->used, append, append_len);
+	buf->used += append_len;
+	buf->free -= append_len;
 	return append_len;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_appendf(php_http_buffer_t *buf,
-		const char *format, ...)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_appendf(php_http_buffer_t *buf, const char *format, ...)
 {
 	va_list argv;
 	char *append;
@@ -145,13 +132,10 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_appendf(php_http_buffer_t *buf,
 	return append_len;
 }
 
-PHP_HTTP_BUFFER_API char *php_http_buffer_data(const php_http_buffer_t *buf,
-		char **into, size_t *len)
+PHP_HTTP_BUFFER_API char *php_http_buffer_data(const php_http_buffer_t *buf, char **into, size_t *len)
 {
 	char *copy = ecalloc(1, buf->used + 1);
-	if (buf->data) {
-		memcpy(copy, buf->data, buf->used);
-	}
+	memcpy(copy, buf->data, buf->used);
 	if (into) {
 		*into = copy;
 	}
@@ -161,8 +145,7 @@ PHP_HTTP_BUFFER_API char *php_http_buffer_data(const php_http_buffer_t *buf,
 	return copy;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_cut(php_http_buffer_t *buf,
-		size_t offset, size_t length)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_cut(php_http_buffer_t *buf, size_t offset, size_t length)
 {
 	if (offset > buf->used) {
 		return 0;
@@ -170,19 +153,15 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_cut(php_http_buffer_t *buf,
 	if (offset + length > buf->used) {
 		length = buf->used - offset;
 	}
-	memmove(buf->data + offset, buf->data + offset + length,
-			buf->used - length - offset);
+	memmove(buf->data + offset, buf->data + offset + length, buf->used - length - offset);
 	buf->used -= length;
 	buf->free += length;
 	return length;
 }
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_fix(
-		php_http_buffer_t *buf)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_fix(php_http_buffer_t *buf)
 {
-	if (	buf->free < 1 &&
-			PHP_HTTP_BUFFER_NOMEM == php_http_buffer_resize_ex(buf, 1, 1, 0)
-	) {
+	if (buf->free < 1 && PHP_HTTP_BUFFER_NOMEM == php_http_buffer_resize_ex(buf, 1, 1, 0)) {
 		return NULL;
 	}
 	buf->data[buf->used] = '\0';
@@ -214,16 +193,14 @@ PHP_HTTP_BUFFER_API void php_http_buffer_free(php_http_buffer_t **buf)
 	}
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_chunk_buffer(php_http_buffer_t **s,
-		const char *data, size_t data_len, char **chunk, size_t chunk_size)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_chunk_buffer(php_http_buffer_t **s, const char *data, size_t data_len, char **chunk, size_t chunk_size)
 {
 	php_http_buffer_t *storage;
 	
 	*chunk = NULL;
 	
 	if (!*s) {
-		*s = php_http_buffer_init_ex(NULL, chunk_size << 1,
-				chunk_size ? PHP_HTTP_BUFFER_INIT_PREALLOC : 0);
+		*s = php_http_buffer_init_ex(NULL, chunk_size << 1, chunk_size ? PHP_HTTP_BUFFER_INIT_PREALLOC : 0);
 	}
 	storage = *s;
 	
@@ -246,15 +223,13 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_chunk_buffer(php_http_buffer_t **s,
 	return 0;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_chunked_output(php_http_buffer_t **s,
-		const char *data, size_t data_len, size_t chunk_len,
-		php_http_buffer_pass_func_t passout, void *opaque)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_chunked_output(php_http_buffer_t **s, const char *data, size_t data_len, size_t chunk_len, php_http_buffer_pass_func_t passout, void *opaque TSRMLS_DC)
 {
 	char *chunk = NULL;
 	size_t passed = 0, got = 0;
 
 	while ((got = php_http_buffer_chunk_buffer(s, data, data_len, &chunk, chunk_len))) {
-		if (PHP_HTTP_BUFFER_PASS0 == passout(opaque, chunk, got)) {
+		if (PHP_HTTP_BUFFER_PASS0 == passout(opaque, chunk, got TSRMLS_CC)) {
 			PTR_SET(chunk, NULL);
 			return PHP_HTTP_BUFFER_PASS0;
 		}
@@ -272,19 +247,15 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_chunked_output(php_http_buffer_t **s,
 	return passed;
 }
 
-PHP_HTTP_BUFFER_API ssize_t php_http_buffer_passthru(php_http_buffer_t **s, size_t chunk_size,
-		php_http_buffer_pass_func_t passin, void *passin_arg,
-		php_http_buffer_pass_func_t passon, void *passon_arg)
+PHP_HTTP_BUFFER_API ssize_t php_http_buffer_passthru(php_http_buffer_t **s, size_t chunk_size, php_http_buffer_pass_func_t passin, void *passin_arg, php_http_buffer_pass_func_t passon, void *passon_arg TSRMLS_DC)
 {
-	size_t passed_on = 0, passed_in;
-
-	passed_in = php_http_buffer_chunked_input(s, chunk_size, passin, passin_arg);
+	size_t passed_on = 0, passed_in = php_http_buffer_chunked_input(s, chunk_size, passin, passin_arg TSRMLS_CC);
 
 	if (passed_in == PHP_HTTP_BUFFER_PASS0) {
 		return passed_in;
 	}
 	if (passed_in || (*s)->used) {
-		passed_on = passon(passon_arg, (*s)->data, (*s)->used);
+		passed_on = passon(passon_arg, (*s)->data, (*s)->used TSRMLS_CC);
 
 		if (passed_on == PHP_HTTP_BUFFER_PASS0) {
 			return passed_on;
@@ -298,20 +269,18 @@ PHP_HTTP_BUFFER_API ssize_t php_http_buffer_passthru(php_http_buffer_t **s, size
 	return passed_on - passed_in;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_chunked_input(php_http_buffer_t **s,
-		size_t chunk_size, php_http_buffer_pass_func_t passin, void *opaque)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_chunked_input(php_http_buffer_t **s, size_t chunk_size, php_http_buffer_pass_func_t passin, void *opaque TSRMLS_DC)
 {
 	php_http_buffer_t *str;
 	size_t passed;
 
-	if (UNEXPECTED(!*s)) {
-		*s = php_http_buffer_init_ex(NULL, chunk_size,
-				chunk_size ? PHP_HTTP_BUFFER_INIT_PREALLOC : 0);
+	if (!*s) {
+		*s = php_http_buffer_init_ex(NULL, chunk_size, chunk_size ? PHP_HTTP_BUFFER_INIT_PREALLOC : 0);
 	}
 	str = *s;
 
 	php_http_buffer_resize(str, chunk_size);
-	passed = passin(opaque, str->data + str->used, chunk_size);
+	passed = passin(opaque, str->data + str->used, chunk_size TSRMLS_CC);
 
 	if (passed != PHP_HTTP_BUFFER_PASS0) {
 		str->used += passed;
@@ -325,8 +294,7 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_chunked_input(php_http_buffer_t **s,
 
 #ifdef PHP_HTTP_BUFFER_EXTENDED
 
-PHP_HTTP_BUFFER_API int php_http_buffer_cmp(php_http_buffer_t *left,
-		php_http_buffer_t *right)
+PHP_HTTP_BUFFER_API int php_http_buffer_cmp(php_http_buffer_t *left, php_http_buffer_t *right)
 {
 	if (left->used > right->used) {
 		return -1;
@@ -337,8 +305,7 @@ PHP_HTTP_BUFFER_API int php_http_buffer_cmp(php_http_buffer_t *left,
 	}
 }
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_copy(
-		const php_http_buffer_t *from, php_http_buffer_t *to)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_copy(const php_http_buffer_t *from, php_http_buffer_t *to)
 {
 	int free_to = !to;
 
@@ -354,8 +321,7 @@ PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_copy(
 	return to;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_insert(php_http_buffer_t *buf,
-		const char *insert, size_t insert_len, size_t offset)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_insert(php_http_buffer_t *buf, const char *insert, size_t insert_len, size_t offset)
 {
 	if (PHP_HTTP_BUFFER_NOMEM == php_http_buffer_resize(buf, insert_len)) {
 		return PHP_HTTP_BUFFER_NOMEM;
@@ -367,8 +333,7 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_insert(php_http_buffer_t *buf,
 	return insert_len;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_insertf(php_http_buffer_t *buf,
-		size_t offset, const char *format, ...)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_insertf(php_http_buffer_t *buf, size_t offset, const char *format, ...)
 {
 	va_list argv;
 	char *insert;
@@ -387,8 +352,7 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_insertf(php_http_buffer_t *buf,
 	return insert_len;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_prepend(php_http_buffer_t *buf,
-		const char *prepend, size_t prepend_len)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_prepend(php_http_buffer_t *buf, const char *prepend, size_t prepend_len)
 {
 	if (PHP_HTTP_BUFFER_NOMEM == php_http_buffer_resize(buf, prepend_len)) {
 		return PHP_HTTP_BUFFER_NOMEM;
@@ -400,8 +364,7 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_prepend(php_http_buffer_t *buf,
 	return prepend_len;
 }
 
-PHP_HTTP_BUFFER_API size_t php_http_buffer_prependf(php_http_buffer_t *buf,
-		const char *format, ...)
+PHP_HTTP_BUFFER_API size_t php_http_buffer_prependf(php_http_buffer_t *buf, const char *format, ...)
 {
 	va_list argv;
 	char *prepend;
@@ -420,20 +383,13 @@ PHP_HTTP_BUFFER_API size_t php_http_buffer_prependf(php_http_buffer_t *buf,
 	return prepend_len;
 }
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_sub(
-		const php_http_buffer_t *buf, size_t offset, size_t length)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_sub(const php_http_buffer_t *buf, size_t offset, size_t length)
 {
 	if (offset >= buf->used) {
 		return NULL;
 	} else {
-		php_http_buffer_t *sub;
-		size_t need = 1 + ((length + offset) > buf->used ?
-				(buf->used - offset) : (length - offset));
-		unsigned flags = buf->pmem ? PHP_HTTP_BUFFER_INIT_PERSISTENT : 0;
-
-		sub = php_http_buffer_init_ex(NULL, need,
-				PHP_HTTP_BUFFER_INIT_PREALLOC | flags);
-
+		size_t need = 1 + ((length + offset) > buf->used ? (buf->used - offset) : (length - offset));
+		php_http_buffer_t *sub = php_http_buffer_init_ex(NULL, need, PHP_HTTP_BUFFER_INIT_PREALLOC | (buf->pmem ? PHP_HTTP_BUFFER_INIT_PERSISTENT:0));
 		if (sub) {
 			if (PHP_HTTP_BUFFER_NOMEM == php_http_buffer_append(sub, buf->data + offset, need)) {
 				php_http_buffer_free(&sub);
@@ -445,8 +401,7 @@ PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_sub(
 	}
 }
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_right(
-		const php_http_buffer_t *buf, size_t length)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_right(const php_http_buffer_t *buf, size_t length)
 {
 	if (length < buf->used) {
 		return php_http_buffer_sub(buf, buf->used - length, length);
@@ -456,8 +411,7 @@ PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_right(
 }
 
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_merge_va(
-		php_http_buffer_t *buf, unsigned argc, va_list argv)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_merge_va(php_http_buffer_t *buf, unsigned argc, va_list argv)
 {
 	unsigned i = 0;
 	buf = php_http_buffer_init(buf);
@@ -474,8 +428,7 @@ PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_merge_va(
 	return buf;
 }
 
-PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_merge_ex(
-		php_http_buffer_t *buf, unsigned argc, ...)
+PHP_HTTP_BUFFER_API php_http_buffer_t *php_http_buffer_merge_ex(php_http_buffer_t *buf, unsigned argc, ...)
 {
 	va_list argv;
 	php_http_buffer_t *ret;
