@@ -12,7 +12,7 @@
 
 #include "php_http_api.h"
 
-php_http_info_t *php_http_info_init(php_http_info_t *i)
+php_http_info_t *php_http_info_init(php_http_info_t *i TSRMLS_DC)
 {
 	if (!i) {
 		i = emalloc(sizeof(*i));
@@ -30,11 +30,11 @@ void php_http_info_dtor(php_http_info_t *i)
 			PTR_SET(PHP_HTTP_INFO(i).request.method, NULL);
 			PTR_SET(PHP_HTTP_INFO(i).request.url, NULL);
 			break;
-
+		
 		case PHP_HTTP_RESPONSE:
 			PTR_SET(PHP_HTTP_INFO(i).response.status, NULL);
 			break;
-
+		
 		default:
 			break;
 	}
@@ -49,7 +49,7 @@ void php_http_info_free(php_http_info_t **i)
 	}
 }
 
-void php_http_info_to_string(php_http_info_t *info, char **str, size_t *len, const char *eol)
+void php_http_info_to_string(php_http_info_t *info, char **str, size_t *len, const char *eol TSRMLS_DC)
 {
 	char *tmp = NULL;
 
@@ -74,12 +74,11 @@ void php_http_info_to_string(php_http_info_t *info, char **str, size_t *len, con
 				info->http.info.request.method&&!strcasecmp(info->http.info.request.method,"CONNECT")?(
 				info->http.info.request.url?php_http_url_authority_to_string(info->http.info.request.url, &(tmp), NULL):"0"):(
 				info->http.info.request.url?php_http_url_to_string(info->http.info.request.url, &(tmp), NULL, 0):"/"),
-				info->http.version.major||info->http.version.minor?info->http.version.major:1,
+				info->http.version.major||info->http.version.major?info->http.version.major:1,
 				info->http.version.major||info->http.version.minor?info->http.version.minor:1,
 				eol);
 	} else if (info->type == PHP_HTTP_RESPONSE){
-		*len = spprintf(str, 0, "HTTP/%u.%u %d%s%s%s",
-				info->http.version.major||info->http.version.minor?info->http.version.major:1,
+		*len = spprintf(str, 0, "HTTP/%u.%u %d%s%s%s", info->http.version.major||info->http.version.major?info->http.version.major:1,
 				info->http.version.major||info->http.version.minor?info->http.version.minor:1,
 				info->http.info.response.code?info->http.info.response.code:200,
 				info->http.info.response.status&&*info->http.info.response.status ? " ":"",
@@ -87,32 +86,32 @@ void php_http_info_to_string(php_http_info_t *info, char **str, size_t *len, con
 				eol);
 	}
 
-	PTR_FREE(tmp);
+	STR_FREE(tmp);
 }
 
-php_http_info_t *php_http_info_parse(php_http_info_t *info, const char *pre_header)
+php_http_info_t *php_http_info_parse(php_http_info_t *info, const char *pre_header TSRMLS_DC)
 {
 	const char *end, *http, *off;
 	zend_bool free_info = !info;
-
+	
 	/* sane parameter */
 	if ((!pre_header) || (!*pre_header)) {
 		return NULL;
 	}
-
+	
 	/* where's the end of the line */
 	if (!(end = php_http_locate_eol(pre_header, NULL))) {
 		end = pre_header + strlen(pre_header);
 	}
-
+	
 	/* there must be HTTP/1.x in the line */
 	if (!(http = php_http_locate_str(pre_header, end - pre_header, "HTTP/", lenof("HTTP/")))) {
 		return NULL;
 	}
+	
+	info = php_http_info_init(info TSRMLS_CC);
 
-	info = php_http_info_init(info);
-
-	if (!php_http_version_parse(&info->http.version, http)) {
+	if (!php_http_version_parse(&info->http.version, http TSRMLS_CC)) {
 		if (free_info) {
 			php_http_info_free(&info);
 		}
@@ -144,9 +143,9 @@ php_http_info_t *php_http_info_parse(php_http_info_t *info, const char *pre_head
 	/* is response */
 	if (pre_header == http) {
 		const char *status = NULL, *code = off;
-
+		
 		info->type = PHP_HTTP_RESPONSE;
-		while (code < end && ' ' == *code) ++code;
+		while (' ' == *code) ++code;
 		if (end > code) {
 			/* rfc7230#3.1.2 The status-code element is a 3-digit integer code */
 			PHP_HTTP_INFO(info).response.code = 100*(*code++ - '0');
@@ -168,14 +167,14 @@ php_http_info_t *php_http_info_parse(php_http_info_t *info, const char *pre_head
 		} else {
 			PHP_HTTP_INFO(info).response.status = NULL;
 		}
-
+		
 		return info;
 	}
-
+	
 	/* is request */
 	else if (*(http - 1) == ' ' && (!*off || *off == '\r' || *off == '\n')) {
 		const char *url = strchr(pre_header, ' ');
-
+		
 		info->type = PHP_HTTP_REQUEST;
 		if (url && http > url) {
 			size_t url_len = url - pre_header;
@@ -188,9 +187,9 @@ php_http_info_t *php_http_info_parse(php_http_info_t *info, const char *pre_head
 			if (http > url) {
 				/* CONNECT presents an authority only */
 				if (strcasecmp(PHP_HTTP_INFO(info).request.method, "CONNECT")) {
-					PHP_HTTP_INFO(info).request.url = php_http_url_parse(url, http - url, PHP_HTTP_URL_STDFLAGS);
+					PHP_HTTP_INFO(info).request.url = php_http_url_parse(url, http - url, PHP_HTTP_URL_STDFLAGS TSRMLS_CC);
 				} else {
-					PHP_HTTP_INFO(info).request.url = php_http_url_parse_authority(url, http - url, PHP_HTTP_URL_STDFLAGS);
+					PHP_HTTP_INFO(info).request.url = php_http_url_parse_authority(url, http - url, PHP_HTTP_URL_STDFLAGS TSRMLS_CC);
 				}
 				if (!PHP_HTTP_INFO(info).request.url) {
 					PTR_SET(PHP_HTTP_INFO(info).request.method, NULL);
@@ -204,7 +203,7 @@ php_http_info_t *php_http_info_parse(php_http_info_t *info, const char *pre_head
 			PHP_HTTP_INFO(info).request.method = NULL;
 			PHP_HTTP_INFO(info).request.url = NULL;
 		}
-
+		
 		return info;
 	}
 
