@@ -510,7 +510,8 @@ zend_class_entry *php_http_message_get_class_entry(void)
 }
 
 static zval *php_http_message_object_read_prop(zval *object, zval *member, int type, void **cache_slot, zval *rv);
-static void php_http_message_object_write_prop(zval *object, zval *member, zval *value, void **cache_slot);
+
+static PHP_WRITE_PROP_HANDLER_TYPE php_http_message_object_write_prop(zval *object, zval *member, zval *value, void **cache_slot);
 
 static zend_object_handlers php_http_message_object_handlers;
 static HashTable php_http_message_object_prophandlers;
@@ -776,7 +777,7 @@ ZEND_RESULT_CODE php_http_message_object_set_body(php_http_message_object_t *msg
 		case IS_RESOURCE:
 			php_stream_from_zval_no_verify(s, zbody);
 			if (!s) {
-				php_http_throw(unexpected_val, "The stream is not a valid resource", NULL);
+				php_http_throw(unexpected_val, "The stream is not a valid resource");
 				return FAILURE;
 			}
 
@@ -894,6 +895,13 @@ void php_http_message_object_free(zend_object *object)
 	zend_object_std_dtor(object);
 }
 
+#if PHP_VERSION_ID >= 70400
+static zval *php_http_message_object_get_prop_ptr(zval *object, zval *member, int type, void **cache_slot)
+{
+	return NULL;
+}
+#endif
+
 static zval *php_http_message_object_read_prop(zval *object, zval *member, int type, void **cache_slot, zval *tmp)
 {
 	zval *return_value;
@@ -923,7 +931,7 @@ static zval *php_http_message_object_read_prop(zval *object, zval *member, int t
 	return return_value;
 }
 
-static void php_http_message_object_write_prop(zval *object, zval *member, zval *value, void **cache_slot)
+static PHP_WRITE_PROP_HANDLER_TYPE php_http_message_object_write_prop(zval *object, zval *member, zval *value, void **cache_slot)
 {
 	php_http_message_object_t *obj = PHP_HTTP_OBJ(NULL, object);
 	php_http_message_object_prophandler_t *handler;
@@ -938,6 +946,7 @@ static void php_http_message_object_write_prop(zval *object, zval *member, zval
 	}
 
 	zend_string_release(member_name);
+	PHP_WRITE_PROP_HANDLER_RETURN(value);
 }
 
 static HashTable *php_http_message_object_get_debug_info(zval *object, int *is_temp)
@@ -1076,7 +1085,7 @@ static PHP_METHOD(HttpMessage, __construct)
 			php_http_buffer_init_ex(&buf, 0x1000, PHP_HTTP_BUFFER_INIT_PREALLOC);
 			if (PHP_HTTP_MESSAGE_PARSER_STATE_FAILURE == php_http_message_parser_parse_stream(&p, &buf, s, flags, &msg)) {
 				if (!EG(exception)) {
-					php_http_throw(bad_message, "Could not parse message from stream", NULL);
+					php_http_throw(bad_message, "Could not parse message from stream");
 				}
 			}
 			php_http_buffer_dtor(&buf);
@@ -1084,7 +1093,7 @@ static PHP_METHOD(HttpMessage, __construct)
 		}
 
 		if (!msg && !EG(exception)) {
-			php_http_throw(bad_message, "Empty message received from stream", NULL);
+			php_http_throw(bad_message, "Empty message received from stream");
 		}
 	} else if (zmessage) {
 		zend_string *zs_msg = zval_get_string(zmessage);
@@ -1092,7 +1101,7 @@ static PHP_METHOD(HttpMessage, __construct)
 		msg = php_http_message_parse(NULL, zs_msg->val, zs_msg->len, greedy);
 
 		if (!msg && !EG(exception)) {
-			php_http_throw(bad_message, "Could not parse message: %.*s", MIN(25, zs_msg->len), zs_msg->val);
+			php_http_throw(bad_message, "Could not parse message: %.*s", (int) MIN(25, zs_msg->len), zs_msg->val);
 		}
 		zend_string_release(zs_msg);
 	}
@@ -1468,7 +1477,7 @@ static PHP_METHOD(HttpMessage, getResponseCode)
 		PHP_HTTP_MESSAGE_OBJECT_INIT(obj);
 
 		if (obj->message->type != PHP_HTTP_RESPONSE) {
-			php_error_docref(NULL, E_WARNING, "http\\Message is not if type response");
+			php_error_docref(NULL, E_WARNING, "http\\Message is not of type response");
 			RETURN_FALSE;
 		}
 
@@ -1492,7 +1501,7 @@ static PHP_METHOD(HttpMessage, setResponseCode)
 	PHP_HTTP_MESSAGE_OBJECT_INIT(obj);
 
 	if (obj->message->type != PHP_HTTP_RESPONSE) {
-		php_http_throw(bad_method_call, "http\\Message is not of type response", NULL);
+		php_http_throw(bad_method_call, "http\\Message is not of type response");
 		return;
 	}
 
@@ -1544,7 +1553,7 @@ static PHP_METHOD(HttpMessage, setResponseStatus)
 	PHP_HTTP_MESSAGE_OBJECT_INIT(obj);
 
 	if (obj->message->type != PHP_HTTP_RESPONSE) {
-		php_http_throw(bad_method_call, "http\\Message is not of type response", NULL);
+		php_http_throw(bad_method_call, "http\\Message is not of type response");
 	}
 
 	PTR_SET(obj->message->http.info.response.status, estrndup(status, status_len));
@@ -1589,12 +1598,12 @@ static PHP_METHOD(HttpMessage, setRequestMethod)
 	PHP_HTTP_MESSAGE_OBJECT_INIT(obj);
 
 	if (obj->message->type != PHP_HTTP_REQUEST) {
-		php_http_throw(bad_method_call, "http\\Message is not of type request", NULL);
+		php_http_throw(bad_method_call, "http\\Message is not of type request");
 		return;
 	}
 
 	if (method_len < 1) {
-		php_http_throw(invalid_arg, "Cannot set http\\Message's request method to an empty string", NULL);
+		php_http_throw(invalid_arg, "Cannot set http\\Message's request method to an empty string");
 		return;
 	}
 
@@ -1645,7 +1654,7 @@ static PHP_METHOD(HttpMessage, setRequestUrl)
 	PHP_HTTP_MESSAGE_OBJECT_INIT(obj);
 
 	if (obj->message->type != PHP_HTTP_REQUEST) {
-		php_http_throw(bad_method_call, "http\\Message is not of type request", NULL);
+		php_http_throw(bad_method_call, "http\\Message is not of type request");
 		return;
 	}
 
@@ -1655,7 +1664,7 @@ static PHP_METHOD(HttpMessage, setRequestUrl)
 
 	if (url && php_http_url_is_empty(url)) {
 		php_http_url_free(&url);
-		php_http_throw(invalid_arg, "Cannot set http\\Message's request url to an empty string", NULL);
+		php_http_throw(invalid_arg, "Cannot set http\\Message's request url to an empty string");
 	} else if (url) {
 		PTR_SET(obj->message->http.info.request.url, url);
 	}
@@ -1675,7 +1684,7 @@ static PHP_METHOD(HttpMessage, getParentMessage)
 	PHP_HTTP_MESSAGE_OBJECT_INIT(obj);
 
 	if (!obj->message->parent) {
-		php_http_throw(unexpected_val, "http\\Message has not parent message", NULL);
+		php_http_throw(unexpected_val, "http\\Message has no parent message");
 		return;
 	}
 
@@ -1832,7 +1841,7 @@ static PHP_METHOD(HttpMessage, prepend)
 	for (msg[0] = obj->message; msg[0]; msg[0] = msg[0]->parent) {
 		for (msg[1] = prepend_obj->message; msg[1]; msg[1] = msg[1]->parent) {
 			if (msg[0] == msg[1]) {
-				php_http_throw(unexpected_val, "Cannot prepend a message located within the same message chain", NULL);
+				php_http_throw(unexpected_val, "Cannot prepend a message located within the same message chain");
 				return;
 			}
 		}
@@ -1892,7 +1901,7 @@ static PHP_METHOD(HttpMessage, splitMultipartBody)
 	PHP_HTTP_MESSAGE_OBJECT_INIT(obj);
 
 	if (!php_http_message_is_multipart(obj->message, &boundary)) {
-		php_http_throw(bad_method_call, "http\\Message is not a multipart message", NULL);
+		php_http_throw(bad_method_call, "http\\Message is not a multipart message");
 		return;
 	}
 
@@ -1993,7 +2002,7 @@ static PHP_METHOD(HttpMessage, current)
 }
 
 static zend_function_entry php_http_message_methods[] = {
-	PHP_ME(HttpMessage, __construct,        ai_HttpMessage___construct,        ZEND_ACC_PUBLIC|ZEND_ACC_CTOR)
+	PHP_ME(HttpMessage, __construct,        ai_HttpMessage___construct,        ZEND_ACC_PUBLIC)
 	PHP_ME(HttpMessage, getBody,            ai_HttpMessage_getBody,            ZEND_ACC_PUBLIC)
 	PHP_ME(HttpMessage, setBody,            ai_HttpMessage_setBody,            ZEND_ACC_PUBLIC)
 	PHP_ME(HttpMessage, addBody,            ai_HttpMessage_addBody,            ZEND_ACC_PUBLIC)
@@ -2062,7 +2071,11 @@ PHP_MINIT_FUNCTION(http_message)
 	php_http_message_object_handlers.read_property = php_http_message_object_read_prop;
 	php_http_message_object_handlers.write_property = php_http_message_object_write_prop;
 	php_http_message_object_handlers.get_debug_info = php_http_message_object_get_debug_info;
+#if PHP_VERSION_ID >= 70400
+	php_http_message_object_handlers.get_property_ptr_ptr = php_http_message_object_get_prop_ptr;
+#else
 	php_http_message_object_handlers.get_property_ptr_ptr = NULL;
+#endif
 	php_http_message_object_handlers.get_gc = php_http_message_object_get_gc;
 
 	zend_class_implements(php_http_message_class_entry, 3, spl_ce_Countable, zend_ce_serializable, zend_ce_iterator);
