@@ -214,7 +214,7 @@ size_t php_http_message_body_append(php_http_message_body_t *body, const char *b
 	written = php_stream_write(s, buf, len);
 
 	if (written != len) {
-		php_error_docref(NULL, E_WARNING, "Failed to append %zu bytes to body; wrote %zu", len, written);
+		php_error_docref(NULL, E_WARNING, "Failed to append %zu bytes to body; wrote %zu", len, written == (size_t) -1 ? 0 : written);
 	}
 
 	return len;
@@ -330,7 +330,7 @@ static inline char *format_key(php_http_arrkey_t *key, const char *prefix) {
 	} else if (key->key) {
 		new_key = estrdup(key->key->val);
 	} else {
-		new_key = estrdup("");
+		spprintf(&new_key, 0, "%lu", key->h);
 	}
 
 	return new_key;
@@ -392,14 +392,17 @@ static ZEND_RESULT_CODE add_recursive_files(php_http_message_body_t *body, const
 			ZEND_HASH_FOREACH_KEY_VAL_IND(files, key.h, key.key, val)
 			{
 				if (Z_TYPE_P(val) == IS_ARRAY || Z_TYPE_P(val) == IS_OBJECT) {
-					char *str = format_key(&key, name);
+					char *str = key.key ? format_key(&key, name) : NULL;
+					const char *prefix = str ? str : name;
 
-					if (SUCCESS != add_recursive_files(body, str, HASH_OF(val))) {
+					if (SUCCESS != add_recursive_files(body, prefix, HASH_OF(val))) {
 						efree(str);
 						HT_UNPROTECT_RECURSION(files);
 						return FAILURE;
 					}
-					efree(str);
+					if (str) {
+						efree(str);
+					}
 				}
 			}
 			ZEND_HASH_FOREACH_END();
@@ -907,7 +910,7 @@ PHP_METHOD(HttpMessageBody, stat)
 }
 
 static zend_function_entry php_http_message_body_methods[] = {
-	PHP_ME(HttpMessageBody, __construct,  ai_HttpMessageBody___construct,  ZEND_ACC_PUBLIC|ZEND_ACC_CTOR)
+	PHP_ME(HttpMessageBody, __construct,  ai_HttpMessageBody___construct,  ZEND_ACC_PUBLIC)
 	PHP_ME(HttpMessageBody, __toString,   ai_HttpMessageBody___toString,   ZEND_ACC_PUBLIC)
 	PHP_MALIAS(HttpMessageBody, toString, __toString, ai_HttpMessageBody___toString, ZEND_ACC_PUBLIC)
 	PHP_MALIAS(HttpMessageBody, serialize, __toString, ai_HttpMessageBody___toString, ZEND_ACC_PUBLIC)
