@@ -2035,7 +2035,7 @@ PHP_METHOD(HttpUrl, toArray)
 }
 
 static zend_function_entry php_http_url_methods[] = {
-	PHP_ME(HttpUrl, __construct,  ai_HttpUrl___construct, ZEND_ACC_PUBLIC|ZEND_ACC_CTOR)
+	PHP_ME(HttpUrl, __construct,  ai_HttpUrl___construct, ZEND_ACC_PUBLIC)
 	PHP_ME(HttpUrl, mod,          ai_HttpUrl_mod, ZEND_ACC_PUBLIC)
 	PHP_ME(HttpUrl, toString,     ai_HttpUrl_toString, ZEND_ACC_PUBLIC)
 	ZEND_MALIAS(HttpUrl, __toString, toString, ai_HttpUrl_toString, ZEND_ACC_PUBLIC)
@@ -2088,14 +2088,14 @@ PHP_MINIT_FUNCTION(http_url)
 #endif
 	zend_declare_class_constant_long(php_http_url_class_entry, ZEND_STRL("PARSE_TOPCT"), PHP_HTTP_URL_PARSE_TOPCT);
 
-	INIT_NS_CLASS_ENTRY(ce, "http\\Env", "Url", php_http_url_methods);
-	php_http_env_url_class_entry = zend_register_internal_class_ex(&ce, php_http_url_class_entry);
-
 	zend_declare_class_constant_long(php_http_url_class_entry, ZEND_STRL("IGNORE_ERRORS"), PHP_HTTP_URL_IGNORE_ERRORS);
 	zend_declare_class_constant_long(php_http_url_class_entry, ZEND_STRL("SILENT_ERRORS"), PHP_HTTP_URL_SILENT_ERRORS);
 
 	zend_declare_class_constant_long(php_http_url_class_entry, ZEND_STRL("STDFLAGS"), PHP_HTTP_URL_STDFLAGS);
 
+	INIT_NS_CLASS_ENTRY(ce, "http\\Env", "Url", NULL);
+	php_http_env_url_class_entry = zend_register_internal_class_ex(&ce, php_http_url_class_entry);
+
 	return SUCCESS;
 }
 
